import { Type, INode, ISimpleType, IType, TypeFlags, IContext, IValidationResult, ObjectNode, IAnyType, IChildNodesMap } from "../internal";
export declare class CoreType<C, S, T> extends Type<C, S, T> {
    readonly shouldAttachNode: boolean;
    readonly checker: (value: any) => boolean;
    readonly flags: TypeFlags;
    readonly initializer: (v: any) => any;
    constructor(name: any, flags: TypeFlags, checker: any, initializer?: (v: any) => any);
    describe(): string;
    instantiate(parent: ObjectNode | null, subpath: string, environment: any, snapshot: T): INode;
    createNewInstance(node: INode, childNodes: IChildNodesMap, snapshot: any): any;
    isValidSnapshot(value: any, context: IContext): IValidationResult;
}
/**
 * Creates a type that can only contain a string value.
 * This type is used for string values by default
 *
 * @export
 * @alias types.string
 * @example
 * const Person = types.model({
 *   firstName: types.string,
 *   lastName: "Doe"
 * })
 */
export declare const string: ISimpleType<string>;
/**
 * Creates a type that can only contain a numeric value.
 * This type is used for numeric values by default
 *
 * @export
 * @alias types.number
 * @example
 * const Vector = types.model({
 *   x: types.number,
 *   y: 1.5
 * })
 */
export declare const number: ISimpleType<number>;
/**
 * Creates a type that can only contain an integer value.
 * This type is used for integer values by default
 *
 * @export
 * @alias types.integer
 * @example
 * const Size = types.model({
 *   width: types.integer,
 *   height: 10
 * })
 */
export declare const integer: ISimpleType<number>;
/**
 * Creates a type that can only contain a boolean value.
 * This type is used for boolean values by default
 *
 * @export
 * @alias types.boolean
 * @example
 * const Thing = types.model({
 *   isCool: types.boolean,
 *   isAwesome: false
 * })
 */
export declare const boolean: ISimpleType<boolean>;
/**
 * The type of the value `null`
 *
 * @export
 * @alias types.null
 */
export declare const nullType: ISimpleType<null>;
/**
 * The type of the value `undefined`
 *
 * @export
 * @alias types.undefined
 */
export declare const undefinedType: ISimpleType<undefined>;
/**
 * Creates a type that can only contain a javascript Date value.
 *
 * @export
 * @alias types.Date
 * @example
 * const LogLine = types.model({
 *   timestamp: types.Date,
 * })
 *
 * LogLine.create({ timestamp: new Date() })
 */
export declare const DatePrimitive: IType<number | Date, number, Date>;
export declare function getPrimitiveFactoryFromValue(value: any): ISimpleType<any>;
export declare function isPrimitiveType<C = any, S = any, T = any>(type: IAnyType): type is CoreType<C, S, T>;
